#include <exec/exec.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>
#include <intuition/extensions.h>
#include <graphics/gfx.h>
#include <cybergraphx/cybergraphics.h>

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/cybergraphics.h>
#include <proto/graphics.h>
#include <proto/tinygl.h>

#include "../ref_gl/r_local.h"

#ifndef SA_GammaControl
#define SA_GammaControl (SA_Dummy + 123)
#endif

#ifndef SA_3DSupport
#define SA_3DSupport (SA_Dummy + 127)
#endif

#ifndef SA_GammaRed
#define SA_GammaRed (SA_Dummy + 124)
#endif

#ifndef SA_GammaBlue
#define SA_GammaBlue (SA_Dummy + 125)
#endif

#ifndef SA_GammaGreen
#define SA_GammaGreen (SA_Dummy + 126)
#endif

struct IntuitionBase *IntuitionBase;
struct Library *TinyGLBase;
struct GfxBase *GfxBase;
struct Library *CyberGfxBase;

void *pointermem;

GLContext *__tglContext;
static int glctx;

struct Screen *screen;
struct Window *window;

int GLimp_Init(void *hinstance, void *wndproc)
{
	return qtrue;
}

void GLimp_Shutdown(void)
{
	if (glctx)
	{
		if (screen && !(TinyGLBase->lib_Version == 0 && TinyGLBase->lib_Revision < 4))
			glADestroyContextScreen();
		else
			glADestroyContextWindowed();

		glctx = 0;
	}

	if (__tglContext)
	{
		GLClose(__tglContext);
		__tglContext = 0;
	}

	if (window)
	{
		CloseWindow(window);
		window = 0;
	}

	if (pointermem)
	{
		FreeVec(pointermem);
		pointermem = 0;
	}

	if (screen)
	{
		CloseScreen(screen);
		screen = 0;
	}
}

int GLimp_SetMode(int mode, qboolean fullscreen)
{
	int retval = rserr_unknown;
	int depth = 24;
	int r;
	unsigned int width, height;

	width = Cvar_VariableValue("r_width");
	height = Cvar_VariableValue("r_height");

	if (width == 0 || height == 0)
	{
		width = 640;
		height = 480;
	}

	GLimp_Shutdown();

	if (r_colorbits->value != 0)
	{
		depth = r_colorbits->value;
	}

	if (fullscreen)
	{
		ULONG modeid = INVALID_ID;

		if (!(IntuitionBase->LibNode.lib_Version > 50 || (IntuitionBase->LibNode.lib_Version == 50 && IntuitionBase->LibNode.lib_Revision >= 50)))
		{
			modeid = BestCModeIDTags(
				CYBRBIDTG_Depth, depth,
				CYBRBIDTG_NominalWidth, width,
				CYBRBIDTG_NominalHeight, height,
				TAG_DONE);
		}

		if (modeid != INVALID_ID || (IntuitionBase->LibNode.lib_Version > 50 || (IntuitionBase->LibNode.lib_Version == 50 && IntuitionBase->LibNode.lib_Revision >= 50)))
		{
			screen = OpenScreenTags(0,
				modeid!=INVALID_ID?SA_DisplayID:TAG_IGNORE, modeid,
				SA_Width, width,
				SA_Height, height,
				modeid==INVALID_ID?SA_Depth:TAG_IGNORE, depth,
				SA_Quiet, TRUE,
				SA_GammaControl, TRUE,
				SA_3DSupport, TRUE,
				TAG_DONE);

#if 0
			if (screen)
			{
				if (IntuitionBase->LibNode.lib_Version > 50 || (IntuitionBase->LibNode.lib_Version == 50 && IntuitionBase->LibNode.lib_Revision >= 74))
				{
					gammatable = AllocVec(256*3, MEMF_ANY);
					if (gammatable)
						vid_hwgamma_enabled = 1;
				}
			}
#endif
		}
	}

	window = OpenWindowTags(0,
		WA_InnerWidth, width,
		WA_InnerHeight, height,
		WA_Title, "Warsow",
		WA_DragBar, screen?FALSE:TRUE,
		WA_DepthGadget, screen?FALSE:TRUE,
		WA_Borderless, screen?TRUE:FALSE,
		WA_RMBTrap, TRUE,
		screen?WA_PubScreen:TAG_IGNORE, (ULONG)screen,
		WA_Activate, TRUE,
		TAG_DONE);

	if (window)
	{
		pointermem = AllocVec(256, MEMF_ANY|MEMF_CLEAR);
		if (pointermem)
		{
			SetPointer(window, pointermem, 16, 16, 0, 0);

			__tglContext = GLInit();
			if (__tglContext)
			{
				if (screen && !(TinyGLBase->lib_Version == 0 && TinyGLBase->lib_Revision < 4))
				{
					r = glAInitializeContextScreen(screen);
				}
				else
				{
					r = glAInitializeContextWindowed(window);
				}

				if (r != 0)
				{
					glctx = 1;

					glState.width = width;
					glState.height = height;
					glState.fullScreen = screen?1:0;
					VID_NewWindow(width, height);

					return rserr_ok;
				}

				GLClose(__tglContext);
				__tglContext = 0;
			}

			FreeVec(pointermem);
			pointermem = 0;
		}

		CloseWindow(window);
		window = 0;
	}
	else
		retval = rserr_unknown;

	if (screen)
	{
		CloseScreen(screen);
		screen = 0;
	}

	return retval;
}

void GLimp_BeginFrame(void)
{
}

void GLimp_EndFrame(void)
{
	glASwapBuffers();
}

qboolean GLimp_GetGammaRamp(size_t stride, unsigned short *ramp)
{
	return qfalse;
}

void GLimp_SetGammaRamp(size_t stride, unsigned short *ramp)
{
}

