#include <exec/exec.h>

#include <proto/exec.h>

#include "../qcommon/qcommon.h"

extern struct IntuitionBase *IntuitionBase;
extern struct Library *TinyGLBase;
extern struct GfxBase *GfxBase;
extern struct Library *CyberGfxBase;

#define QGL_EXTERN

#define QGL_FUNC(type,name,params) type (APIENTRY * q##name) params;
#define QGL_EXT(type,name,params) type (APIENTRY * q##name) params;
#define QGL_WGL(type,name,params)
#define QGL_WGL_EXT(type,name,params)
#define QGL_GLX(type,name,params)
#define QGL_GLX_EXT(type,name,params)

#include "../ref_gl/qgl.h"

#undef QGL_GLX_EXT
#undef QGL_GLX
#undef QGL_WGL_EXT
#undef QGL_WGL
#undef QGL_EXT
#undef QGL_FUNC

qboolean QGL_Init(const char *drivername)
{
	IntuitionBase = (struct IntuitionBase *)OpenLibrary("intuition.library", 0);
	if (IntuitionBase)
	{
		TinyGLBase = OpenLibrary("tinygl.library", 0);
		if (TinyGLBase)
		{
			GfxBase = (struct GfxBase *)OpenLibrary("graphics.library", 0);
			if (GfxBase)
			{
				CyberGfxBase = OpenLibrary("cybergraphics.library", 0);
				if (CyberGfxBase)
				{
					return qtrue;
				}

				CloseLibrary((struct Library *)GfxBase);
				GfxBase = 0;
			}

			CloseLibrary(TinyGLBase);
			TinyGLBase = 0;
		}

		CloseLibrary((struct Library *)IntuitionBase);
		IntuitionBase = 0;
	}

	return qfalse;
}

void QGL_Shutdown( void )
{
#define QGL_FUNC(type,name,params) (q##name) = NULL;
#define QGL_EXT(type,name,params) (q##name) = NULL;
#define QGL_WGL(type,name,params)
#define QGL_WGL_EXT(type,name,params)
#define QGL_GLX(type,name,params)
#define QGL_GLX_EXT(type,name,params)

#include "../ref_gl/qgl.h"

#undef QGL_GLX_EXT
#undef QGL_GLX
#undef QGL_WGL_EXT
#undef QGL_WGL
#undef QGL_EXT
#undef QGL_FUNC

	if (IntuitionBase)
	{
		CloseLibrary((struct Library *)IntuitionBase);
		CloseLibrary(TinyGLBase);
		CloseLibrary(CyberGfxBase);
		IntuitionBase = 0;
		TinyGLBase = 0;
		CyberGfxBase = 0;
	}
}

void glCopyTexSubImage2D()
{
} 

/* TinyGL stubs */

void myglMultiTexCoord2fARB(GLenum unit, GLfloat s, GLfloat t)
{
	GLMultiTexCoord2fARB(__tglContext, unit, s, t);
}

void myglActiveTextureARB(GLenum unit)
{
	GLActiveTextureARB(__tglContext, unit);
}

void myglClientActiveTextureARB(GLenum unit)
{
	GLClientActiveTextureARB(__tglContext, unit);
}

void myglPointParameterfEXT(GLenum pname, GLfloat param)
{
	GLPointParameterfEXT(__tglContext, pname, param);
}

void myglPointParameterfvEXT(GLenum pname, GLfloat *params)
{
	GLPointParameterfvEXT(__tglContext, pname, params);
}

void myglLockArraysEXT(GLint first, GLint count)
{
	GLLockArraysEXT(__tglContext, first, count);
}

void myglUnlockArraysEXT(void)
{
	GLUnlockArraysEXT(__tglContext);
}

void myglColorTable(GLenum target, GLenum internalformat, GLint width, GLenum format, GLenum type, GLvoid *data)
{
	GLColorTable(__tglContext, target, internalformat, width, format, type, data);
}

void *qglGetProcAddress(const GLubyte *symbol)
{
	if (strcmp(symbol, "glMultiTexCoord2fARB") == 0)
		return (void *)myglMultiTexCoord2fARB;
	else if (strcmp(symbol, "glActiveTextureARB") == 0)
		return (void *)myglActiveTextureARB;
	else if (strcmp(symbol, "glClientActiveTextureARB") == 0)
		return (void *)myglClientActiveTextureARB;
	else if (strcmp(symbol, "glPointParameterfEXT") == 0)
		return (void *)myglPointParameterfEXT;
	else if (strcmp(symbol, "glPointParameterfvEXT") == 0)
		return (void *)myglPointParameterfvEXT;
	else if (strcmp(symbol, "glLockArraysEXT") == 0)
		return (void *)myglLockArraysEXT;
	else if (strcmp(symbol, "glUnlockArraysEXT") == 0)
		return (void *)myglUnlockArraysEXT;
	else if (strcmp(symbol, "glColorTableEXT") == 0)
		return (void *)myglColorTable;

	return 0;
}

