#include <intuition/intuition.h>
#include <dos/dos.h>
#include <dos/dosextens.h>
#include <devices/clipboard.h>

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/dos.h>

#include <sys/stat.h>
#include <sys/time.h>
#include <signal.h>

#undef __MORPHOS_SHAREDLIBS
#include <dlfcn.h>

#include "../qcommon/qcommon.h"

int __stack = 4*1024*1024;

static int consoleoutput;

unsigned int sys_frame_time;

static BPTR findlock;
static struct FileInfoBlock findfib;
char finddir[256];
char findres[512];
char findpattern[514];

unsigned int curtime;

void Sys_ConsoleOutput (char *string)
{
	if (consoleoutput)
		fputs(string, stdout);
}

char *Sys_ConsoleInput(void)
{
	return 0;
}

void Sys_SendKeyEvents (void)
{
#warning Do keyboard stuff here

	sys_frame_time = Sys_Milliseconds();
}

void Sys_AppActivate(void)
{
}

char *Sys_GetClipboardData(void)
{
	struct IOClipReq *clipreq;
	struct MsgPort *clipport;
	unsigned int iffstuff[5];
	char *clipdata = 0;

	clipport = CreateMsgPort();
	if (clipport)
	{
		clipreq = CreateIORequest(clipport, sizeof(*clipreq));
		if (clipreq)
		{
			if (OpenDevice("clipboard.device", 0, (struct IORequest *)clipreq, 0) == 0)
			{
				clipreq->io_Command = CMD_READ;
				clipreq->io_Length = sizeof(iffstuff);
				clipreq->io_Data = (void *)iffstuff;
				clipreq->io_Offset = 0;
				clipreq->io_ClipID = 0;

				DoIO((struct IORequest *)clipreq);

				if (clipreq->io_Error == 0
				 && clipreq->io_Actual == sizeof(iffstuff)
				 && iffstuff[0] == 0x464F524D
				 && iffstuff[2] == 0x46545854
				 && iffstuff[3] == 0x43485253)
				{
					clipdata = malloc(iffstuff[4]+1);
					if (clipdata)
					{
						clipreq->io_Command = CMD_READ;
						clipreq->io_Length = iffstuff[4];
						clipreq->io_Data = clipdata;

						DoIO((struct IORequest *)clipreq);

						if (clipreq->io_Error != 0 || clipreq->io_Actual != iffstuff[4])
						{
							free(clipdata);
							clipdata = 0;
						}
						else
						{
							clipdata[iffstuff[4]] = 0;
						}
					}
				}

				do
				{
					clipreq->io_Command = CMD_READ;
					clipreq->io_Length = 0x100000;
					clipreq->io_Data = 0;

					DoIO((struct IORequest *)clipreq);
				} while(clipreq->io_Error == 0 && clipreq->io_Actual == 0x100000);

				CloseDevice((struct IORequest *)clipreq);
			}

			DeleteIORequest((struct IORequest *)clipreq);
		}

		DeleteMsgPort(clipport);
	}

	return clipdata;
}

char *Sys_FindFirst(char *path, unsigned musthave, unsigned canhave)
{
	char *pattern;
	char pattrans[256];
	int i, j;

	if (findlock)
		UnLock(findlock);

	strlcpy(finddir, path, sizeof(finddir));

	pattern = strrchr(path, '/');
	if (pattern)
	{
		finddir[((unsigned int)(pattern-path))] = 0;
		pattern++;
	}

	for(i=0,j=0;i<sizeof(pattrans)-1 && *pattern;i++,j++)
	{
		if (pattern[j] == '*')
		{
			if (i < sizeof(pattrans)-2)
			{
				pattrans[i] = '#';
				pattrans[i+1] = '?';
				i++;
			}
			else
				pattrans[i] = 0;
		}
		else
			pattrans[i] = pattern[j];
	}

	pattrans[i] = 0;

	findlock = Lock(finddir, ACCESS_READ);
	if (findlock)
	{
		if (Examine(findlock, &findfib))
		{
			if (ParsePatternNoCase(pattrans, findpattern, sizeof(findpattern)) >= 0)
				return Sys_FindNext(musthave, canhave);
		}

		UnLock(findlock);
		findlock = 0;
	}

	return 0;
}

char *Sys_FindNext(unsigned musthave, unsigned canhave)
{
	while(ExNext(findlock, &findfib))
	{
		if (MatchPatternNoCase(findpattern, findfib.fib_FileName))
		{
			snprintf(findres, sizeof(findres), "%s/%s", finddir, findfib.fib_FileName);
			return findres;
		}
	}

	return 0;
}

void Sys_FindClose(void)
{
	if (findlock)
	{
		findlock = UnLock(findlock);
		findlock = 0;
	}
}

void Sys_Init(void)
{
}

void Sys_Quit(void)
{
	CL_Shutdown();
	Qcommon_Shutdown();

	if (findlock)
	{
		UnLock(findlock);
		findlock = 0;
	}

	exit(0);
}

static void ErrorMessage(char *string)
{
	char msg[4096];

	snprintf(msg, sizeof(msg), "%s", string);

	if (consoleoutput)
		fprintf(stderr, "%s\n", msg);
	else
	{
		struct EasyStruct es;

		if (*msg && msg[strlen(msg)-1] == '\n')
			msg[strlen(msg)-1] = 0;

		es.es_StructSize = sizeof(es);
		es.es_Flags = 0;
		es.es_Title = "Quake 2";
		es.es_TextFormat = msg;
		es.es_GadgetFormat = "Quit";

		EasyRequest(0, &es, 0, 0);
	}
}

void Sys_Error(char *error, ...)
{
	va_list va;
	char msg[4096];

	strcpy(msg, "Sys_Error: ");

	va_start(va, error);
	vsnprintf(msg+strlen(msg), sizeof(msg)-strlen(msg), error, va);
	va_end(va);

	ErrorMessage(msg);

	Sys_Quit();
}

static void *gamefiles[3];
static const char *sofilenames[3] = { "game.so", "cgame.so", "ui.so" };
static const char *getapifunc[3] = { "GetGameAPI", "GetCGameAPI", "GetUIAPI" };

void *Sys_LoadGameLibrary(gamelib_t gamelib, void *parms)
{
	int (*warsow_so_init)(void);
	void (*warsow_so_deinit)(void);
	void *(*GetGameAPI)(void *);
	void *ret;
	char *searchpath;
	char path[256];

	searchpath = 0;

	while((searchpath = FS_NextPath(searchpath)))
	{

		snprintf(path, sizeof(path), "%s%s%s", searchpath[0]!='.'?searchpath:"", searchpath[0]&&searchpath[0]!='.'?"/":"", sofilenames[gamelib]);

		gamefiles[gamelib] = dlopen(path, RTLD_NOW);
		if (gamefiles[gamelib])
		{
			warsow_so_init = dlsym(gamefiles[gamelib], "warsow_so_init");
			warsow_so_deinit = dlsym(gamefiles[gamelib], "warsow_so_deinit");
			if (warsow_so_init && warsow_so_init())
			{
				GetGameAPI = dlsym(gamefiles[gamelib], getapifunc[gamelib]);
				if (GetGameAPI && (ret = GetGameAPI(parms)))
				{
					return ret;
				}

				if (warsow_so_deinit)
					warsow_so_deinit();
			}
			dlclose(gamefiles[gamelib]);
			gamefiles[gamelib] = 0;
		}
	}

	return 0;
}

void Sys_UnloadGameLibrary(gamelib_t gamelib)
{
	void (*warsow_so_deinit)(void);

	if (gamefiles[gamelib])
	{
		warsow_so_deinit = dlsym(gamefiles[gamelib], "warsow_so_deinit");
		if (warsow_so_deinit)
			warsow_so_deinit();

		dlclose(gamefiles[gamelib]);
		gamefiles[gamelib] = 0;
	}
}

unsigned int Sys_Milliseconds(void)
{
	struct timeval tp;
	struct timezone tzp;
	static unsigned int secbase;

	gettimeofday(&tp, &tzp);
	
	if (!secbase)
	{
		secbase = tp.tv_sec;
		curtime = tp.tv_usec/1000;
		return curtime;
	}

	curtime = (tp.tv_sec - secbase)*1000 + tp.tv_usec/1000;

	return curtime;
}

void Sys_Mkdir(const char *path)
{
	mkdir(path, 0777);
}

char *Sys_GetHomeDirectory()
{
	return 0;
}

int main(int argc, char **argv)
{
	extern struct WBStartup *_WBenchMsg;

	int time, oldtime, newtime;

	if (!_WBenchMsg)
	{
		consoleoutput = 1;
	}

	signal(SIGINT, (void *)SIG_IGN);

	Qcommon_Init(argc, argv);

	oldtime = Sys_Milliseconds();
	while((SetSignal(0, 0)&SIGBREAKF_CTRL_C) == 0)
	{
		Mem_CheckSentinelsGlobal();
// find time spent rendering last frame
		do
		{
			newtime = Sys_Milliseconds();
			time = newtime - oldtime;
		} while (time < 1);

		Qcommon_Frame (time);

		oldtime = newtime;
	}

	Mem_CheckSentinelsGlobal();
	Sys_Error("CTRL-C\n");
}

